@@ -637,8 +637,8 @@ main_prog (void *closure, int argc, char *argv[])
   master_client = gnome_master_client ();
   g_signal_connect (master_client, "save-yourself",
 		    G_CALLBACK (save_yourself_cb), &data);
-  g_signal_connect_swapped (master_client, "die",
-                            G_CALLBACK (die_cb), &data);
+  g_signal_connect (master_client, "die",
+                    G_CALLBACK (die_cb), &data);
 #endif /* HAVE_GNOME */
 
 #ifdef HAVE_HILDON
