@@ -203,6 +203,8 @@ class GtkUI(glchess.ui.UI):
         self.feedback = feedback
         self._watches = {}
         self.__networkGames = {}
+        self.newGameDialog = None
+        self.loadGameDialog = None
         self.__saveGameDialogs = {}
         self.__joinGameDialogs = []
         
@@ -229,6 +231,7 @@ class GtkUI(glchess.ui.UI):
         # Create the model for the player types
         self.__playerModel = gtk.ListStore(str, str, str)
         iter = self.__playerModel.append()
+        # Translators: Player Type Combo: Player is human controlled
         self.__playerModel.set(iter, 0, '', 1, 'stock_person', 2, _('Human'))
         
         self.__logWindow = log.LogWindow(self._gui.get_widget('log_notebook'))
@@ -357,13 +360,13 @@ class GtkUI(glchess.ui.UI):
         # Set the window title to the name of the game
         if self.view is not None and len(self.view.title) > 0:
             if self.view.needsSaving:
-                # Translators: This is the window title when playing a game that needs saving
+                # Translators: Window title when playing a game that needs saving
                 title = _('Chess - *%(game_name)s') % {'game_name': self.view.title}
             else:
-                # Translators: This is the window title when playing a game that is saved
+                # Translators: Window title when playing a game that is saved
                 title = _('Chess - %(game_name)s') % {'game_name': self.view.title}
         else:
-            # Translators: This is the window title when not playing a game
+            # Translators: Window title when not playing a game
             title = _('Chess')            
         self.mainWindow.set_title(title)
 
@@ -461,6 +464,7 @@ class GtkUI(glchess.ui.UI):
                                    type = gtk.MESSAGE_WARNING,
                                    message_format = title)
         dialog.format_secondary_text("If you don't save the changes to this game will be permanently lost")
+        # Translators: Save Game Dialog: Discard game button
         dialog.add_button(_('Close _without saving'), gtk.RESPONSE_OK)
         dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT)
         dialog.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT)
@@ -575,12 +579,15 @@ class GtkUI(glchess.ui.UI):
         # Enable/disable OpenGL rendering
         elif name == 'show_3d':            
             if value and not chessview.haveGLSupport:
+                # Translators: No 3D Dialog: Title
                 title = _('Unable to enable 3D mode')
                 errors = '\n'.join(chessview.openGLErrors)
+                # Translators: No 3D Dialog: Notification to user that they do not have libraries required to enable 3D.
+                # %(error)s will be replaced with a list of reasons why 3D is not available.
                 description = _("""You are unable to play in 3D mode due to the following problems:
 %(errors)s
 
-Please contact your system administrator to resolve these problems, until then you will be able to play chess in 2D mode.""" % {'errors': errors})
+Please contact your system administrator to resolve these problems, until then you will be able to play chess in 2D mode.""") % {'errors': errors}
                 dialog = gtk.MessageDialog(type = gtk.MESSAGE_WARNING, message_format = title)
                 dialog.format_secondary_text(description)
                 dialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
@@ -784,7 +791,10 @@ Please contact your system administrator to resolve these problems, until then y
 
     def _on_new_game_button_clicked(self, widget):
         """Gtk+ callback"""
-        dialogs.GtkNewGameDialog(self, self.__playerModel)
+        if self.newGameDialog:
+            self.newGameDialog.window.present()
+        else:
+            self.newGameDialog = dialogs.GtkNewGameDialog(self, self.__playerModel)
 
     def _on_join_game_button_clicked(self, widget):
         """Gtk+ callback"""
@@ -792,19 +802,30 @@ Please contact your system administrator to resolve these problems, until then y
 
     def _on_open_game_button_clicked(self, widget):
         """Gtk+ callback"""
-        dialogs.GtkLoadGameDialog(self)
+        if self.loadGameDialog:
+            self.loadGameDialog.window.present()
+        else:
+            self.loadGameDialog = dialogs.GtkLoadGameDialog(self)
         
     def _on_save_game_button_clicked(self, widget):
         """Gtk+ callback"""
         if self.view.feedback.getFileName() is not None:
             self.view.feedback.save()
-        elif not self.__saveGameDialogs.has_key(self.view):
-            self.__saveGameDialogs[self.view] = dialogs.GtkSaveGameDialog(self, self.view)
-            
+            return
+        
+        try:
+            dialog = self.__saveGameDialogs[self.view]
+        except KeyError:
+            dialog = self.__saveGameDialogs[self.view] = dialogs.GtkSaveGameDialog(self, self.view)
+        dialog.window.present()
+
     def _on_save_as_game_button_clicked(self, widget):
         """Gtk+ callback"""
-        if not self.__saveGameDialogs.has_key(self.view):
-            self.__saveGameDialogs[self.view] = dialogs.GtkSaveGameDialog(self, self.view, self.view.feedback.getFileName())
+        try:
+            dialog = self.__saveGameDialogs[self.view]
+        except KeyError:
+            dialog = self.__saveGameDialogs[self.view] = dialogs.GtkSaveGameDialog(self, self.view, self.view.feedback.getFileName())
+        dialog.window.present()
 
     def _on_resign_clicked(self, widget):
         """Gtk+ callback"""
@@ -812,13 +833,20 @@ Please contact your system administrator to resolve these problems, until then y
 
     def _on_claim_draw_clicked(self, widget):
         """Gtk+ callback"""
-        if not self.view.feedback.claimDraw():
-            dialog = gtk.MessageDialog(flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-                                       type = gtk.MESSAGE_WARNING,
-                                       message_format = _("Unable to claim draw"))
-        dialog.format_secondary_text(_("""You may claim a draw when:
+        if self.view.feedback.claimDraw():
+            return
+        
+        # Translators: Draw Dialog: Title
+        title = _("Unable to claim draw")
+        # Translators: Draw Dialog: Notify user why they cannot claim draw
+        message = _("""You may claim a draw when:
 a) The board has been in the same state three times (Three fold repitition)
-b) Fifty moves have occured where no pawn has moved and no piece has been captured (50 move rule)"""))
+b) Fifty moves have occured where no pawn has moved and no piece has been captured (50 move rule)""")
+        
+        dialog = gtk.MessageDialog(flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                                   type = gtk.MESSAGE_WARNING,
+                                   message_format = title)
+        dialog.format_secondary_text(message)
         dialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_ACCEPT)
         dialog.run()
         dialog.destroy()
