@@ -260,9 +260,10 @@ about_cb (GtkAction * action, gpointer data)
   gtk_show_about_dialog (GTK_WINDOW (app),
 			 "name", _("Robots"),
 			 "version", VERSION,
-			 "copyright", "Copyright \xc2\xa9 1998-2007 Mark Rae",
+			 "copyright", "Copyright \xc2\xa9 1998-2008 Mark Rae",
 			 "license", license,
-			 "comments", _("Based on classic BSD Robots."),
+		         "website-label", _("GNOME Games web site"),
+			 "comments", _("Based on classic BSD Robots.\n\nRobots is a part of GNOME Games."),
 			 "authors", authors,
 			 "documenters", documenters,
 			 "translator-credits", _("translator-credits"),
@@ -397,6 +398,7 @@ toggle_toolbar_cb (GtkAction * action, GtkWidget * toolbar)
 
   state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
   if (state) {
+    gtk_widget_hide (toolbar); /* hack to unfocus the toolbar */
     gtk_widget_show (toolbar);
   } else {
     gtk_widget_hide (toolbar);
